#!/bin/bash
#
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

set -e
set -u

DIR="$( cd -P "$( dirname "$0" )" && pwd )"
SYSTEM_EXT_DIR="${DIR}/../ext"
JAVA_OPTIONS=${JAVA_OPTIONS:-}
HELP_MODE=false

if [ ! -z "${JAVA_OPTIONS}" ]; then
  USER_EXT_DIR=$(grep -o '\-Dtinkerpop.ext=\(\([^"][^ ]*\)\|\("[^"]*"\)\)' <<< "${JAVA_OPTIONS}" | cut -f2 -d '=' | xargs -0 echo)
  #If the user dir is specified and (the user dir doesn't exist or the plugins.txt file is empty or doesn't exist) then create it
  if [ ! -z "${USER_EXT_DIR}" -a \( ! -d "${USER_EXT_DIR}" -o ! -s "${USER_EXT_DIR}/plugins.txt" \) ]; then
    mkdir -p "${USER_EXT_DIR}"
    cp -R ${SYSTEM_EXT_DIR}/* ${USER_EXT_DIR}/
  fi
fi

if [ -z ${GREMLIN_CONSOLE_CONF_DIR+x} ]; then
    for dir in "`dirname $0`/../conf" \
               "/etc/dse/graph/gremlin-console/conf"; do
        if [ -r "$dir/remote.yaml" ]; then
            export GREMLIN_CONSOLE_CONF_DIR="$dir"
            break
        fi
    done
    if [ -z ${GREMLIN_CONSOLE_CONF_DIR+x} ]; then
        echo "Could not found a valid configuration folder"
        exit 1
    fi
fi

CP="$GREMLIN_CONSOLE_CONF_DIR"
CP="$CP":$( echo `dirname $0`/../lib/*.jar . | sed 's/ /:/g')

SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
done

CP=$CP:$( find -L "${SYSTEM_EXT_DIR}" "${USER_EXT_DIR:-${SYSTEM_EXT_DIR}}" -mindepth 1 -maxdepth 1 -type d | \
          sort -u | sed 's/$/\/plugin\/*/' | tr '\n' ':' )

export CLASSPATH="${CLASSPATH:-}:$CP"

# Find Java
if [ -z "${JAVA_HOME:-}" ]; then
    JAVA="java -server"
else
    JAVA="$JAVA_HOME/bin/java -server"
fi

# Script debugging is disabled by default, but can be enabled with -l
# TRACE or -l DEBUG or enabled by exporting
# SCRIPT_DEBUG=nonemptystring to gremlin.sh's environment
if [ -z "${SCRIPT_DEBUG:-}" ]; then
    SCRIPT_DEBUG=
fi

INIT_SCRIPT_EXECUTION_FLAG="-i"

# Process options
while getopts ":leh" opt; do
    case "$opt" in
    l) eval GREMLIN_LOG_LEVEL=\$$OPTIND
       OPTIND="$(( $OPTIND + 1 ))"
       if [ "$GREMLIN_LOG_LEVEL" = "TRACE" -o \
            "$GREMLIN_LOG_LEVEL" = "DEBUG" ]; then
	   SCRIPT_DEBUG=y
       fi
       ;;
    e) INIT_SCRIPT_EXECUTION_FLAG="-e"
       ;;
    h) HELP_MODE=true
       ;;
    esac
done

JAVA_OPTIONS="${JAVA_OPTIONS} -Duser.working_dir=${USER_DIR} -Dtinkerpop.ext=${USER_EXT_DIR:-${SYSTEM_EXT_DIR}} -Dlogback.configurationFile=logback-tools.xml"

if [ -n "${GREMLIN_LOG_LEVEL:-}" ]; then
    JAVA_OPTIONS="$JAVA_OPTIONS -Dlogback.root.level=$GREMLIN_LOG_LEVEL"
fi

JAVA_OPTIONS=$(awk -v RS=' ' '!/^$/ {if (!x[$0]++) print}' <<< "${JAVA_OPTIONS}" | grep -v '^$' | paste -sd ' ' -)

if [ -n "$SCRIPT_DEBUG" ]; then
    # in debug mode enable debugging of :install command
    JAVA_OPTIONS="${JAVA_OPTIONS} -Divy.message.logger.level=4 -Dgroovy.grape.report.downloads=true"
    echo "CLASSPATH: $CLASSPATH"
    set -x
fi

if [ x"true" != x"$HELP_MODE" ] ; then
    # Start the JVM, execute the application, and return its exit code
    exec $JAVA $JAVA_OPTIONS org.apache.tinkerpop.gremlin.console.Console $INIT_SCRIPT_EXECUTION_FLAG scripts/dse-init.groovy "$@"
else
    # Start the JVM with all of its options, one of which (we assume)
    # is -h.  Search-and-replace the initial usage line with DSE's usage
    # line (DSE adds an optional [host[:port]] argument).  Also append
    # a paragraph at the end describing these host and port arguments.

    $JAVA $JAVA_OPTIONS org.apache.tinkerpop.gremlin.console.Console $INIT_SCRIPT_EXECUTION_FLAG scripts/dse-init.groovy "$@" 2>&1 | \
        sed '1 s/^[Uu]sage.*/Usage: gremlin-console [host[:port]] [options] [...]/' >&2

    cat >&2 <<EOF

DSE Gremlin Console automatically connects at startup to DSE as configured
in the conf/remote.yaml file. The host and port of this configuration can be
overridden from the command line using the optional host and port arguments.
EOF

    exit 0
fi
